@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
 # Pacewar
-# Copyright (C) 2014 Julian Marchant <onpon4@riseup.net>
+# Copyright (C) 2014, 2015 Julian Marchant <onpon4@riseup.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = "1.5.3"
+__version__ = "1.5.5"
 
 import os
 import math
@@ -31,6 +31,11 @@ import json
 
 import sge
 
+DATA = os.path.join(os.path.dirname(__file__), "data")
+DATA_IMAGES = os.path.join(DATA, "images")
+DATA_MUSIC = os.path.join(DATA, "music")
+DATA_SOUNDS = os.path.join(DATA, "sounds")
+
 VIEW_WIDTH = 1280
 VIEW_HEIGHT = 720
 ROOM_WIDTH = VIEW_WIDTH * 3
@@ -85,7 +90,6 @@ CONFIG_DIR = os.path.join(os.path.expanduser("~"), ".pacewar")
 
 colorblind = False
 points_to_win = 3
-score = 0
 
 ships_lists = {TEAM_RED: [], TEAM_GREEN: []}
 bullets_lists = {TEAM_RED: [], TEAM_GREEN: []}
@@ -110,10 +114,6 @@ player2_js_shoot = (1, "button", 1)
 
 class Game(sge.Game):
 
-    def event_game_start(self):
-        self.scale = None
-        self.mouse.visible = False
-
     def event_step(self, time_passed, delta_mult):
         x = self.width / 2 - meter_sprite.width / 2
         y = 64
@@ -124,7 +124,7 @@ class Game(sge.Game):
 
         if key == "f7":
             colorblind = not colorblind
-            update_meter()
+            self.current_room.update_meter()
         elif key == "f8":
             fname = "screenshot-{}.bmp".format(round(time.time(), 3))
             sge.Sprite.from_screenshot().save(fname)
@@ -150,6 +150,7 @@ class Game(sge.Game):
 class Room(sge.Room):
 
     def event_room_start(self):
+        self.score = 0
         self.started = False
         self.finished = False
         self.multiplayer = False
@@ -159,6 +160,7 @@ class Room(sge.Room):
         self.menu_axes = {}
         self.round_end()
         self.alarms["check_win"] = 5
+        self.update_meter()
 
     def event_step(self, time_passed, delta_mult):
         if not self.started:
@@ -244,11 +246,9 @@ class Room(sge.Room):
                                     sge.game.width / 2 - menu_w / 2, 240)
         elif self.finished:
             if not music.playing:
-                sge.game.start()
+                create_room().start()
 
     def event_alarm(self, alarm_id):
-        global score
-
         if alarm_id == "check_win":
             red_alive = bool(ships_lists[TEAM_RED])
             green_alive = bool(ships_lists[TEAM_GREEN])
@@ -256,12 +256,12 @@ class Room(sge.Room):
             if red_alive and green_alive:
                 self.alarms["check_win"] = 5
             elif green_alive and not red_alive:
-                score += 1
-                update_meter()
+                self.score += 1
+                self.update_meter()
                 self.round_end()
             elif red_alive and not green_alive:
-                score -= 1
-                update_meter()
+                self.score -= 1
+                self.update_meter()
                 self.round_end()
             else:
                 self.round_end()
@@ -289,7 +289,7 @@ class Room(sge.Room):
 
         if self.finished:
             if key == "enter":
-                sge.game.start()
+                create_room().start()
         elif not self.started:
             if key == "up":
                 self.menu_selection -= 1
@@ -316,7 +316,7 @@ class Room(sge.Room):
                 elif self.menu == MENU_START:
                     if self.menu_selection < 6:
                         points_to_win = self.menu_selection + 1
-                        update_meter()
+                        self.update_meter()
                         self.round_start()
                     else:
                         self.menu = MENU_MAIN
@@ -430,7 +430,7 @@ class Room(sge.Room):
         else:
             if key == "escape":
                 sge.Music.stop(fade_time=500)
-                self.start()
+                create_room().start("dissolve", 500)
             elif key == "enter":
                 sge.Music.pause()
                 select_sound.play()
@@ -466,7 +466,57 @@ class Room(sge.Room):
             select_sound.play()
             sge.game.unpause()
         elif key == "escape":
-            sge.game.start()
+            sge.Music.stop()
+            create_room().start()
+
+    def update_meter(self):
+        global meter_sprite
+
+        meter_w = (meter_left_sprite.width + meter_right_sprite.width +
+                   meter_center_sprite.width +
+                   meter_back_sprite.width * points_to_win * 2)
+        if meter_sprite.width != meter_w:
+            meter_sprite = sge.Sprite(width=meter_w, height=16)
+
+        w = meter_back_sprite.width
+        h = meter_back_sprite.height
+
+        green_meter = sge.Sprite(width=(w * points_to_win), height=h)
+        red_meter = sge.Sprite(width=green_meter.width, height=h)
+
+        green_meter.draw_lock()
+        red_meter.draw_lock()
+
+        for i in range(points_to_win):
+            green_meter.draw_sprite(meter_back_sprite, 0, w * i, 0)
+            red_meter.draw_sprite(meter_back_sprite, 0, w * i, 0)
+
+        if self.score > 0:
+            for i in range(self.score):
+                x = w * i
+                green_meter.draw_sprite(meter_sprites[TEAM_GREEN], 0, x, 0)
+                if colorblind:
+                    green_meter.draw_sprite(colorblind_sprites[TEAM_GREEN], 0,
+                                            x + w / 2 - 8, 0)
+        elif self.score < 0:
+            for i in range(abs(self.score)):
+                x = red_meter.width - w * i
+                red_meter.draw_sprite(meter_sprites[TEAM_RED], 0, x, 0)
+                if colorblind:
+                    red_meter.draw_sprite(colorblind_sprites[TEAM_RED], 0,
+                                          x - w / 2 - 8, 0)
+
+        green_meter.draw_unlock()
+        red_meter.draw_unlock()
+
+        x = 0
+        meter_sprite.draw_lock()
+        meter_sprite.draw_clear()
+        for sprite in [meter_left_sprite, red_meter, meter_center_sprite,
+                       green_meter, meter_right_sprite]:
+            meter_sprite.draw_sprite(sprite, 0, x, 0)
+            x += sprite.width
+        meter_sprite.draw_unlock()
 
     def round_start(self):
         global player1
@@ -482,9 +532,9 @@ class Room(sge.Room):
         if not music.playing:
             music.play(loops=None)
 
-        for i in range(max(1, TEAM_SIZE - max(score, 0))):
+        for i in range(max(1, TEAM_SIZE - max(self.score, 0))):
             Ship.create(TEAM_GREEN)
-        for i in range(max(1, TEAM_SIZE + min(score, 0))):
+        for i in range(max(1, TEAM_SIZE + min(self.score, 0))):
             Ship.create(TEAM_RED)
 
         if self.multiplayer:
@@ -540,19 +590,17 @@ class Room(sge.Room):
         self.alarms["check_win"] = 5
 
     def round_end(self):
-        global score
-
         if self.started:
-            if abs(score) < points_to_win:
+            if abs(self.score) < points_to_win:
                 self.alarms["round_end"] = 90
             else:
                 self.finished = True
                 sge.Music.stop(fade_time=5000)
         else:
-            if score:
-                loser = TEAM_RED if score > 0 else TEAM_GREEN
-                score = 0
-                update_meter()
+            if self.score:
+                loser = TEAM_RED if self.score > 0 else TEAM_GREEN
+                self.score = 0
+                self.update_meter()
                 for i in range(TEAM_SIZE // 2):
                     Ship.create(loser)
             else:
@@ -587,8 +635,7 @@ class Room(sge.Room):
                                   sge.game.height / 2, width=sge.game.width,
                                   height=sge.game.height,
                                   color=sge.Color("white"),
-                                  halign=sge.ALIGN_CENTER,
-                                  valign=sge.ALIGN_MIDDLE)
+                                  halign="center", valign="middle")
 
             # Refresh
             sge.game.refresh()
@@ -641,8 +688,7 @@ class Room(sge.Room):
                                   sge.game.height / 2, width=sge.game.width,
                                   height=sge.game.height,
                                   color=sge.Color("white"),
-                                  halign=sge.ALIGN_CENTER,
-                                  valign=sge.ALIGN_MIDDLE)
+                                  halign="center", valign="middle")
 
             # Refresh
             sge.game.refresh()
@@ -1256,81 +1302,41 @@ def create_nebula(num, z, scroll_rate):
         sprite.draw_sprite(nebula_sprite, 0, x, y)
 
     return sge.BackgroundLayer(sprite, 0, 0, z, xscroll_rate=scroll_rate,
-                               yscroll_rate=scroll_rate, xrepeat=False,
-                               yrepeat=False)
+                               yscroll_rate=scroll_rate)
 
 
-def update_meter():
-    global meter_sprite
-
-    meter_w = (meter_left_sprite.width + meter_right_sprite.width +
-               meter_center_sprite.width +
-               meter_back_sprite.width * points_to_win * 2)
-    if meter_sprite.width != meter_w:
-        meter_sprite = sge.Sprite(width=meter_w, height=16)
-
-    w = meter_back_sprite.width
-    h = meter_back_sprite.height
-
-    green_meter = sge.Sprite(width=(w * points_to_win), height=h)
-    red_meter = sge.Sprite(width=green_meter.width, height=h)
-
-    green_meter.draw_lock()
-    red_meter.draw_lock()
-
-    for i in range(points_to_win):
-        green_meter.draw_sprite(meter_back_sprite, 0, w * i, 0)
-        red_meter.draw_sprite(meter_back_sprite, 0, w * i, 0)
-
-    if score > 0:
-        for i in range(score):
-            x = w * i
-            green_meter.draw_sprite(meter_sprites[TEAM_GREEN], 0, x, 0)
-            if colorblind:
-                green_meter.draw_sprite(colorblind_sprites[TEAM_GREEN], 0,
-                                        x + w / 2 - 8, 0)
-    elif score < 0:
-        for i in range(abs(score)):
-            x = red_meter.width - w * i
-            red_meter.draw_sprite(meter_sprites[TEAM_RED], 0, x, 0)
-            if colorblind:
-                red_meter.draw_sprite(colorblind_sprites[TEAM_RED], 0,
-                                      x - w / 2 - 8, 0)
-
-    green_meter.draw_unlock()
-    red_meter.draw_unlock()
-
-    x = 0
-    meter_sprite.draw_lock()
-    meter_sprite.draw_clear()
-    for sprite in [meter_left_sprite, red_meter, meter_center_sprite,
-                   green_meter, meter_right_sprite]:
-        meter_sprite.draw_sprite(sprite, 0, x, 0)
-        x += sprite.width
-    meter_sprite.draw_unlock()
+def create_room():
+    views = [sge.View(ROOM_WIDTH // 2 - VIEW_WIDTH // 2,
+                      ROOM_HEIGHT // 2 - VIEW_HEIGHT // 2, width=VIEW_WIDTH,
+                      height=VIEW_HEIGHT)]
+    return Room(width=ROOM_WIDTH, height=ROOM_HEIGHT, views=views,
+                background=background)
 
 
 # Create Game object
 Game(width=1280, height=720, scale=SCALE, scale_smooth=True, fps=30,
      delta=True, delta_min=15, window_text="Pacewar",
-     window_icon="Spaceship15B.png")
+     window_icon=os.path.join(DATA_IMAGES, "Spaceship15B.png"))
 
 # Load sprites
-r1_sprite = sge.Sprite("Spaceship14", origin_x=83, origin_y=154, bbox_x=-17,
-                       bbox_y=-17, bbox_width=33, bbox_height=33)
-g1_sprite = sge.Sprite("Spaceship14B", origin_x=83, origin_y=154, bbox_x=-17,
-                       bbox_y=-17, bbox_width=33, bbox_height=33)
-r2_sprite = sge.Sprite("Spaceship15", origin_x=80, origin_y=91, bbox_x=-16,
-                       bbox_y=-16, bbox_width=32, bbox_height=32)
-g2_sprite = sge.Sprite("Spaceship15B", origin_x=80, origin_y=91, bbox_x=-16,
-                       bbox_y=-16, bbox_width=32, bbox_height=32)
-r3_sprite = sge.Sprite("Spaceship16", origin_x=69, origin_y=92, bbox_x=-14,
-                       bbox_y=-14, bbox_width=28, bbox_height=28)
-g3_sprite = sge.Sprite("Spaceship16B", origin_x=69, origin_y=92, bbox_x=-14,
-                       bbox_y=-14, bbox_width=28, bbox_height=28)
-e1_sprite = sge.Sprite("Exhaust14", origin_x=31, origin_y=-65, fps=60)
-e2_sprite = sge.Sprite("Exhaust15", origin_x=35, origin_y=-44, fps=60)
-e3_sprite = sge.Sprite("Exhaust16", origin_x=14, origin_y=-48, fps=60)
+r1_sprite = sge.Sprite("Spaceship14", DATA_IMAGES, origin_x=83, origin_y=154,
+                       bbox_x=-17, bbox_y=-17, bbox_width=33, bbox_height=33)
+g1_sprite = sge.Sprite("Spaceship14B", DATA_IMAGES, origin_x=83, origin_y=154,
+                       bbox_x=-17, bbox_y=-17, bbox_width=33, bbox_height=33)
+r2_sprite = sge.Sprite("Spaceship15", DATA_IMAGES, origin_x=80, origin_y=91,
+                       bbox_x=-16, bbox_y=-16, bbox_width=32, bbox_height=32)
+g2_sprite = sge.Sprite("Spaceship15B", DATA_IMAGES, origin_x=80, origin_y=91,
+                       bbox_x=-16, bbox_y=-16, bbox_width=32, bbox_height=32)
+r3_sprite = sge.Sprite("Spaceship16", DATA_IMAGES, origin_x=69, origin_y=92,
+                       bbox_x=-14, bbox_y=-14, bbox_width=28, bbox_height=28)
+g3_sprite = sge.Sprite("Spaceship16B", DATA_IMAGES, origin_x=69, origin_y=92,
+                       bbox_x=-14, bbox_y=-14, bbox_width=28, bbox_height=28)
+e1_sprite = sge.Sprite("Exhaust14", DATA_IMAGES, origin_x=31, origin_y=-65,
+                       fps=60)
+e2_sprite = sge.Sprite("Exhaust15", DATA_IMAGES, origin_x=35, origin_y=-44,
+                       fps=60)
+e3_sprite = sge.Sprite("Exhaust16", DATA_IMAGES, origin_x=14, origin_y=-48,
+                       fps=60)
 
 ship_sprites = {TEAM_RED: [r1_sprite, r2_sprite, r3_sprite],
                 TEAM_GREEN: [g1_sprite, g2_sprite, g3_sprite]}
@@ -1338,36 +1344,40 @@ exhaust_sprites = {id(r1_sprite): e1_sprite, id(g1_sprite): e1_sprite,
                    id(r2_sprite): e2_sprite, id(g2_sprite): e2_sprite,
                    id(r3_sprite): e3_sprite, id(g3_sprite): e3_sprite}
 
-explosion_sprite = sge.Sprite("explosion", origin_x=64, origin_y=64, fps=30)
-bullet_sprites = {TEAM_RED: sge.Sprite("bullet_red", origin_x=8, origin_y=16),
-                  TEAM_GREEN: sge.Sprite("bullet_green", origin_x=8,
-                                         origin_y=16)}
-stars_sprite = sge.Sprite("Stars")
-nebula_sprites = [sge.Sprite("Nebula1"), sge.Sprite("Nebula2"),
-                  sge.Sprite("Nebula3")]
-target_sprite = sge.Sprite("target", width=80, height=80, origin_x=40,
-                           origin_y=40)
-logo_sprite = sge.Sprite("logo", origin_x=321)
-colorblind_sprites = {TEAM_RED: sge.Sprite("colorblind_red"),
-                      TEAM_GREEN: sge.Sprite("colorblind_green")}
-font_sprite = sge.Sprite("font")
-font_selected_sprite = sge.Sprite("font_selected")
-meter_left_sprite = sge.Sprite("meter_left")
-meter_right_sprite = sge.Sprite("meter_right")
-meter_center_sprite = sge.Sprite("meter_center")
-meter_back_sprite = sge.Sprite("meter_back")
-meter_sprites = {TEAM_RED: sge.Sprite("meter_red", origin_x=37),
-                 TEAM_GREEN: sge.Sprite("meter_green")}
+explosion_sprite = sge.Sprite("explosion", DATA_IMAGES, origin_x=64,
+                              origin_y=64, fps=30)
+bullet_sprites = {TEAM_RED: sge.Sprite("bullet_red", DATA_IMAGES, origin_x=8,
+                                       origin_y=16),
+                  TEAM_GREEN: sge.Sprite("bullet_green", DATA_IMAGES,
+                                         origin_x=8, origin_y=16)}
+stars_sprite = sge.Sprite("Stars", DATA_IMAGES)
+nebula_sprites = [sge.Sprite("Nebula1", DATA_IMAGES),
+                  sge.Sprite("Nebula2", DATA_IMAGES),
+                  sge.Sprite("Nebula3", DATA_IMAGES)]
+target_sprite = sge.Sprite("target", DATA_IMAGES, width=80, height=80,
+                           origin_x=40, origin_y=40)
+logo_sprite = sge.Sprite("logo", DATA_IMAGES, origin_x=321)
+colorblind_sprites = {TEAM_RED: sge.Sprite("colorblind_red", DATA_IMAGES),
+                      TEAM_GREEN: sge.Sprite("colorblind_green", DATA_IMAGES)}
+font_sprite = sge.Sprite("font", DATA_IMAGES)
+font_selected_sprite = sge.Sprite("font_selected", DATA_IMAGES)
+meter_left_sprite = sge.Sprite("meter_left", DATA_IMAGES)
+meter_right_sprite = sge.Sprite("meter_right", DATA_IMAGES)
+meter_center_sprite = sge.Sprite("meter_center", DATA_IMAGES)
+meter_back_sprite = sge.Sprite("meter_back", DATA_IMAGES)
+meter_sprites = {TEAM_RED: sge.Sprite("meter_red", DATA_IMAGES, origin_x=37),
+                 TEAM_GREEN: sge.Sprite("meter_green", DATA_IMAGES)}
 meter_w = (meter_left_sprite.width + meter_right_sprite.width +
            meter_center_sprite.width +
            meter_back_sprite.width * points_to_win * 2)
 meter_sprite = sge.Sprite(width=meter_w, height=16)
-update_meter()
 
 # Load backgrounds
 layers = []
 layers.append(sge.BackgroundLayer(stars_sprite, 0, 0, -1000, xscroll_rate=0.05,
-                                  yscroll_rate=0.01))
+                                  yscroll_rate=0.01, repeat_left=True,
+                                  repeat_right=True, repeat_up=True,
+                                  repeat_down=True))
 layers.append(create_nebula(15, -100, 0.1))
 layers.append(create_nebula(30, -50, 0.5))
 layers.append(create_nebula(5, 5, 1))
@@ -1375,13 +1385,13 @@ layers.append(create_nebula(5, 5, 1))
 background = sge.Background(layers, sge.Color("black"))
 
 # Load sounds
-shoot_sound = sge.Sound("shoot.wav")
-explode_sound = sge.Sound("explode.wav")
-dissipate_sound = sge.Sound("dissipate.ogg")
-select_sound = sge.Sound("select.ogg", volume=50)
+shoot_sound = sge.Sound(os.path.join(DATA_SOUNDS, "shoot.wav"))
+explode_sound = sge.Sound(os.path.join(DATA_SOUNDS, "explode.wav"))
+dissipate_sound = sge.Sound(os.path.join(DATA_SOUNDS, "dissipate.ogg"))
+select_sound = sge.Sound(os.path.join(DATA_SOUNDS, "select.ogg"), volume=50)
 
 # Load music
-music = sge.Music("DST-RailJet-LongSeamlessLoop.ogg")
+music = sge.Music(os.path.join(DATA_MUSIC, "DST-RailJet-LongSeamlessLoop.ogg"))
 
 # Load fonts
 chars = [' ', '!', '"', '#', '$', '%', '&', "'", '(', ')', '*', '+', ',',
@@ -1395,18 +1405,11 @@ chars = [' ', '!', '"', '#', '$', '%', '&', "'", '(', ')', '*', '+', ',',
 menu_font = sge.Font.from_sprite(font_sprite, chars, size=24)
 selection_font = sge.Font.from_sprite(font_selected_sprite, chars, size=24)
 
-# Create objects
-objects = []
-
-# Create views
-views = [sge.View(ROOM_WIDTH // 2 - VIEW_WIDTH // 2,
-                  ROOM_HEIGHT // 2 - VIEW_HEIGHT // 2, width=VIEW_WIDTH,
-                  height=VIEW_HEIGHT)]
-
 # Create room
-sge.game.start_room = Room(objects=objects, width=ROOM_WIDTH,
-                           height=ROOM_HEIGHT, views=views,
-                           background=background)
+sge.game.start_room = create_room()
+
+sge.game.scale = None
+sge.game.mouse.visible = False
 
 if not os.path.exists(CONFIG_DIR):
     os.makedirs(CONFIG_DIR)
